import 'package:flutter/material.dart';
import 'package:date_tab_picker/datePicker/src/date_picker.dart';

import 'datePicker/src/date_picker_theme.dart';
import 'datePicker/src/i18n/date_picker_i18n.dart';
import 'date_util.dart';

class DateTabPicker extends StatefulWidget {
  /// 以当前日期为起始点，依次向前或向后展示的日期数，正为向后，负为向前
  final int daysCount;

  /// tabBar的默认背景颜色
  final Color backgroundColor;

  /// tabBar的定制配置项
  final TabBarConfig tabConfig;

  /// 日期选择器的定制配置项
  final DatePickerConfig pickerConfig;

  /// 日期选择中后的回调函数
  final Function(DateTime) onDatePicked;

  /// tabBar每一项item组件的构建函数
  final Widget Function(DateTime, bool) tabBuilder;

  /// 右侧日历组件
  final Widget calendar;

  DateTabPicker(
      {this.daysCount = 7,
      this.backgroundColor = Colors.white,
      this.tabConfig,
      this.pickerConfig,
      this.onDatePicked,
      this.tabBuilder,
      this.calendar});
  @override
  _DateTabPickerState createState() => _DateTabPickerState();
}

class _DateTabPickerState extends State<DateTabPicker>
    with TickerProviderStateMixin {
  TabController tabController;

  List<DateTime> tabs;

  int curIndex = 0;

  void initState() {
    super.initState();
    initTabs();
  }

  initTabs() {
    int _daysCount = widget.daysCount;
    tabs = List.generate(_daysCount.abs(), (index) {
      DateTime today = new DateTime.now();
      DateTime _date = _daysCount > 0
          ? today.add(new Duration(days: index))
          : today.subtract(new Duration(days: index));
      return _date;
    });
    tabController = TabController(length: tabs.length, vsync: this);
    tabController.addListener(() {
      setState(() {
        curIndex = tabController.index;
      });
    });
  }

  showDatePicker() {
    DatePicker.showDatePicker(context,
        pickerTheme: widget.pickerConfig.pickerTheme,
        locale: widget.pickerConfig.locale,
        minDateTime: widget.pickerConfig.minDateTime,
        maxDateTime: widget.pickerConfig.maxDateTime,
        onConfirm: (DateTime dateTime, List<int> days) async {
      int containIdx = tabs.indexWhere((t) =>
          DateUtil.formatDate(t, format: DateFormats.mo_d) ==
          DateUtil.formatDate(dateTime, format: DateFormats.mo_d));
      if (containIdx > -1) {
        tabController.animateTo(containIdx,
            duration: Duration(milliseconds: 500), curve: Curves.easeIn);
      } else {
        tabs.add(dateTime);
        tabs.sort((a, b) => widget.daysCount > 0
            ? a.millisecondsSinceEpoch - b.millisecondsSinceEpoch
            : b.millisecondsSinceEpoch - a.millisecondsSinceEpoch);
        // tabController = TabController(
        //     length: tabs.length, initialIndex: curIndex, vsync: this);
        tabController = TabController(length: tabs.length, vsync: this);
        tabController.addListener(() {
          setState(() {
            curIndex = tabController.index;
          });
        });
        int nextIdx = tabs.indexWhere((t) =>
            DateUtil.formatDate(t, format: DateFormats.mo_d) ==
            DateUtil.formatDate(dateTime, format: DateFormats.mo_d));
        setState(() {});
        await Future.delayed(Duration(milliseconds: 100));
        tabController.animateTo(nextIdx,
            duration: Duration(milliseconds: 500), curve: Curves.easeIn);
      }
      if (widget.onDatePicked != null) {
        widget.onDatePicked(dateTime);
      }
    });
  }

  @override
  void dispose() {
    super.dispose();
    tabController?.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: widget.backgroundColor,
      child: Row(
        children: <Widget>[
          Expanded(
              child: Container(
            child: TabBar(
                controller: tabController,
                isScrollable: widget.tabConfig?.isScrollable ?? true,
                indicatorSize: widget.tabConfig?.indicatorSize ??
                    TabBarIndicatorSize.label,
                indicatorColor:
                    widget.tabConfig?.indicatorColor ?? Colors.transparent,
                onTap: (int index) {
                  if (widget.onDatePicked != null) {
                    widget.onDatePicked(tabs[index]);
                  }
                },
                tabs: List.generate(
                    tabs.length,
                    (index) =>
                        widget.tabBuilder(tabs[index], index == curIndex))),
          )),
          GestureDetector(
              onTap: () {
                showDatePicker();
              },
              child: widget.calendar),
        ],
      ),
    );
  }
}

class TabBarConfig {
  /// tabBar是否可滚动
  final bool isScrollable;

  /// tabBar下划线指示器的大小参考
  final TabBarIndicatorSize indicatorSize;

  /// tabBar下划线指示器的颜色
  final Color indicatorColor;

  /// tabBar每一项的内间距
  final EdgeInsetsGeometry indicatorPadding;
  TabBarConfig(
      {this.isScrollable = true,
      this.indicatorSize = TabBarIndicatorSize.label,
      this.indicatorColor,
      this.indicatorPadding});
}

class DatePickerConfig {
  /// 日历选择器的主题风格
  final DateTimePickerTheme pickerTheme;

  /// 日历选择器的语言
  final DateTimePickerLocale locale;

  /// 日历最小起始日期
  final DateTime minDateTime;

  /// 日历最大截止日期
  final DateTime maxDateTime;
  DatePickerConfig(
      {this.pickerTheme = DateTimePickerTheme.Default,
      this.locale = DateTimePickerLocale.zh_cn,
      this.minDateTime,
      this.maxDateTime});
}
